{% extends "base.html" %}

{% block title_tab %}Privacy policy{% endblock %}
{% block title_main %}Politica per la privacy{% endblock %}

{% block content %}
    <p>
        La politica per la privacy sarà stilata quando il bot sarà terminato.
    </p>
{% endblock %}
