{% extends "base.html" %}

{% block title_tab %}Errore{% endblock %}
{% block title_main %}Errore{% endblock %}

{% block content %}
    <p>
        Si è verificato un errore {{ when | safe }}.
    </p>
    <p>
        {{ details | safe }}
    </p>
    <p>
        {{ tip | safe }}
    </p>
{% endblock %}